==> v1.0.0.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/visual/service/VisualDataService.java <==
/**
 * 
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.VisualDto;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataPermission;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualDataMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 * @author wookvn
 *
 */
@Service
public class VisualDataService extends ServiceImpl<VisualDataMapper, VisualData> {

	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private VisualDataPermissionService visualDataPermissionService;

	/**
	 * 
	 * @param userId
	 * @return
	 */
	public List<Integer> classifyIdListPerm(Long userId){
		return baseMapper.classifyIdListPerm(userId);
	}
	/**
	 * 
	 * @param visualData
	 * @return
	 */
	@Transactional
	public Integer doSaveOrUpdate(VisualDto visualDto, BladeUser bladeUser){
		VisualData visualData = visualDto.getVisualData();
		Long userId = SecureUtil.getUserId();
		String deptId = SecureUtil.getDeptId();
		if(visualData.getId() == null) { //新增
			visualData.setCreateTime(LocalDateTime.now());
			visualData.setCreateUser(userId);
			visualData.setCreateDept(Func.toLong(deptId, 0L));
		}else {
			visualData.setUpdateTime(LocalDateTime.now());
			visualData.setUpdateUser(userId);
		}
		//1.新增或修改
		boolean ret = this.saveOrUpdate(visualData);
		if(ret) {
			List<ComponentDto> componentList = visualDto.getComponentList();
			//2.新增组件和组件字段
			visualComponentService.batchSaveComponent(visualData.getId(), componentList, bladeUser);
			
			//3.仅添加大屏时：权限组处理【前端在编辑时，不显示权限组选择】
			if(CollectionUtil.isNotEmpty(visualDto.getDataPermIdList())) {
				List<VisualDataPermission> dataList = new ArrayList<>(visualDto.getDataPermIdList().size());
				visualDto.getDataPermIdList().forEach(t -> {
					VisualDataPermission entity = new VisualDataPermission();
					entity.setCreateBy(bladeUser.getNickName());
					entity.setCreateTime(new Date());
					entity.setCreateUserId(bladeUser.getUserId());
					entity.setDataPermId(t);
//					entity.setVisualClassifyId(visualDataPermDto.getVisualClassifyId());
					entity.setVisualId(visualData.getId());
					dataList.add(entity);
				});
				visualDataPermissionService.saveBatch(dataList);
			}
		}
		return visualData.getId();
	}
	
	/**
	 * 删除
	 * @param visualId
	 */
	@Transactional
	public void doRemove(List<Integer> idList) {
		baseMapper.deleteBatchIds(idList);
		visualComponentService.remove(Wrappers.<VisualComponent>lambdaQuery().in(VisualComponent::getVisualId, idList));
		visualDataPermissionService.remove(Wrappers.<VisualDataPermission>lambdaQuery().in(VisualDataPermission::getVisualId, idList));
	}
	
	/**
	 * 复制普通
	 * @param visualId
	 * @param visualCategoryId
	 * @param bladeUser
	 * @return
	 */
	public Integer doCopyGeneral(Integer visualId, Integer visualCategoryId, BladeUser bladeUser) {
		List<Integer> newVisualIdList = _doCopy(Arrays.asList(visualId), visualCategoryId, null, null, VisualTypeEnum.GENERAL, bladeUser);
		return CollectionUtil.isNotEmpty(newVisualIdList) ? newVisualIdList.get(0) : null;
	}
	
	/**
	 * 复制幻灯片
	 * @param visualIdList
	 * @param visualCategoryId
	 * @param bladeUser
	 * @return
	 */
	public List<Integer> doCopyPowerPoint(List<Integer> visualIdList, Integer powerPointId, BladeUser bladeUser){
		return _doCopy(visualIdList, null, null, powerPointId, VisualTypeEnum.POWER_POINT, bladeUser);
	}
	
	/**
	 * 复制报告
	 * @param visualIdList
	 * @param reportInfoId
	 * @param bladeUser
	 * @return
	 */
	public List<Integer> doCopyReport(List<Integer> visualIdList, Integer reportInfoId, BladeUser bladeUser) {
		return _doCopy(visualIdList, null, reportInfoId, null, VisualTypeEnum.REPORT, bladeUser);
	}
	
	@Transactional
	private List<Integer> _doCopy(List<Integer> visualIdList, Integer visualCategoryId, Integer reportInfoId, Integer powerPointId, VisualTypeEnum visualType, BladeUser bladeUser) {
		List<Integer> newVisualIdList = new ArrayList<>();
		//批量添加大屏
		List<VisualData> visualDataList = baseMapper.selectBatchIds(visualIdList);
		List<VisualComponent> visualComponentList = new ArrayList<VisualComponent>();
		visualDataList.forEach(item -> {
			final int oldVisualId = item.getId();
			item.setId(null);
			item.setPowerPointId(powerPointId);
			item.setRefVisualId(oldVisualId);
			item.setVisualType(visualType);
			item.setCreateTime(LocalDateTime.now());
			item.setHistoryReportId(reportInfoId);
			item.setCreateUser(bladeUser.getUserId());
			item.setCreateDept(Func.toLong(bladeUser.getDeptId()));
			item.setUpdateTime(LocalDateTime.now());
			item.setUpdateUser(bladeUser.getUserId());
			if(visualCategoryId != null) {
				item.setCategory(visualCategoryId);
			}
			int ret = baseMapper.insert(item);
			if(ret > 0) { //插入成功
				newVisualIdList.add(item.getId());
				//查询旧组件列表
				List<VisualComponent> componetList = visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId, oldVisualId));
				if(componetList != null && !componetList.isEmpty()) {
					componetList.stream().forEach(f -> {
						f.setId(null);
						f.setVisualId(item.getId());
						f.setCreateBy(bladeUser.getNickName());
						f.setCreateTime(new Date());
						f.setCreateUserId(bladeUser.getUserId());
					});
					visualComponentList.addAll(componetList);
				}
			}
		});
		//批量添加组件列表
		if(CollectionUtil.isNotEmpty(visualComponentList)) {
			visualComponentService.saveBatch(visualComponentList);
		}
		return newVisualIdList;
	}
}

==> v1.1.1.tar.gz: platomix-gmetry-bi/platomix-gmetry-bi-api/src/main/java/org/springblade/visual/service/VisualDataService.java <==
/**
 * 
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.VisualDto;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataPermission;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualDataMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 * @author wookvn
 *
 */
@Service
public class VisualDataService extends ServiceImpl<VisualDataMapper, VisualData> {

	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private VisualDataPermissionService visualDataPermissionService;

	/**
	 * 
	 * @param userId
	 * @return
	 */
	public List<Integer> classifyIdListPerm(Long userId){
		return baseMapper.classifyIdListPerm(userId);
	}
	/**
	 * 
	 * @param visualData
	 * @return
	 */
	@Transactional
	public Integer doSaveOrUpdate(VisualDto visualDto, BladeUser bladeUser){
		VisualData visualData = visualDto.getVisualData();
		Long userId = SecureUtil.getUserId();
		String deptId = SecureUtil.getDeptId();
		if(visualData.getId() == null) { //新增
			visualData.setCreateTime(LocalDateTime.now());
			visualData.setCreateUser(userId);
			visualData.setCreateDept(Func.toLong(deptId, 0L));
		}else {
			visualData.setUpdateTime(LocalDateTime.now());
			visualData.setUpdateUser(userId);
		}
		//1.新增或修改
		boolean ret = this.saveOrUpdate(visualData);
		if(ret) {
			List<ComponentDto> componentList = visualDto.getComponentList();
			//2.新增组件和组件字段
			visualComponentService.batchSaveComponent(visualData.getId(), componentList, bladeUser);
			
			//3.仅添加大屏时：权限组处理【前端在编辑时，不显示权限组选择】
			if(CollectionUtil.isNotEmpty(visualDto.getDataPermIdList())) {
				List<VisualDataPermission> dataList = new ArrayList<>(visualDto.getDataPermIdList().size());
				visualDto.getDataPermIdList().forEach(t -> {
					VisualDataPermission entity = new VisualDataPermission();
					entity.setCreateBy(bladeUser.getNickName());
					entity.setCreateTime(new Date());
					entity.setCreateUserId(bladeUser.getUserId());
					entity.setDataPermId(t);
					entity.setVisualId(visualData.getId());
					dataList.add(entity);
				});
				visualDataPermissionService.saveBatch(dataList);
			}
		}
		return visualData.getId();
	}
	
	/**
	 * 删除
	 * @param visualId
	 */
	@Transactional
	public void doRemove(List<Integer> idList) {
		baseMapper.deleteBatchIds(idList);
		visualComponentService.remove(Wrappers.<VisualComponent>lambdaQuery().in(VisualComponent::getVisualId, idList));
		visualDataPermissionService.remove(Wrappers.<VisualDataPermission>lambdaQuery().in(VisualDataPermission::getVisualId, idList));
	}
	
	/**
	 * 复制幻灯片
	 * @param visualIdList
	 * @param visualCategoryId
	 * @param bladeUser
	 * @return <旧大屏ID, 新大屏ID>
	 */
	public List<Integer> doCopyPowerPoint(Set<Integer> visualIdSet, Integer powerPointId, BladeUser bladeUser){
		return doCopy(visualIdSet, null, null, powerPointId, VisualTypeEnum.POWER_POINT, bladeUser, "");
	}
	
	/**
	 * 复制报告
	 * @param visualIdList
	 * @param reportInfoId
	 * @param bladeUser
	 * @return <旧大屏ID, 新大屏ID>
	 */
	public List<Integer> doCopyReport(Set<Integer> visualIdSet, Integer reportInfoId, BladeUser bladeUser) {
		return doCopy(visualIdSet, null, reportInfoId, null, VisualTypeEnum.REPORT, bladeUser, "");
	}
	
	/**
	 * @param visualIdSet      - 原大屏ID不能有重复
	 * @param visualCategoryId
	 * @param reportInfoId
	 * @param powerPointId
	 * @param visualType
	 * @param bladeUser
	 * @return <新大屏ID列表>
	 */
	@Transactional
	public List<Integer> doCopy(Set<Integer> visualIdSet, Integer visualCategoryId, Integer reportInfoId, Integer powerPointId, VisualTypeEnum visualType, BladeUser bladeUser, String titlePrefix) {
		List<Integer> visualIdList = new ArrayList<>();
		//批量添加大屏
		List<VisualData> visualDataList = baseMapper.selectBatchIds(visualIdSet);
		List<VisualComponent> visualComponentList = new ArrayList<VisualComponent>();
		visualDataList.forEach(item -> {
			final int oldVisualId = item.getId();
			item.setId(null);
			item.setTitle(titlePrefix + item.getTitle());
			item.setRefVisualId(oldVisualId);
			item.setVisualType(visualType);
			item.setCreateTime(LocalDateTime.now());
			item.setHistoryReportId(reportInfoId);
			item.setCreateUser(bladeUser.getUserId());
			item.setCreateDept(Func.toLong(bladeUser.getDeptId()));
			item.setUpdateTime(LocalDateTime.now());
			item.setUpdateUser(bladeUser.getUserId());
			item.setOrderIndex(9999);
			if(visualCategoryId != null) {
				item.setCategory(visualCategoryId);
			}
			if(powerPointId != null) {
				item.setPowerPointId(powerPointId);
			}
			int ret = baseMapper.insert(item);
			if(ret > 0) { //插入成功
				visualIdList.add(item.getId());
				//查询旧组件列表
				List<VisualComponent> componetList = visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId, oldVisualId));
				if(componetList != null && !componetList.isEmpty()) {
					componetList.stream().forEach(f -> {
						f.setId(null);
						f.setVisualId(item.getId());
						f.setCreateBy(bladeUser.getNickName());
						f.setCreateTime(new Date());
						f.setCreateUserId(bladeUser.getUserId());
					});
					visualComponentList.addAll(componetList);
				}
			}
		});
		//批量添加组件列表
		if(CollectionUtil.isNotEmpty(visualComponentList)) {
			visualComponentService.saveBatch(visualComponentList);
		}
		return visualIdList;
	}
}
